//
// Copyright 2006 Johannes Hofmann <Johannes.Hofmann@gmx.de>
//
// This software may be used and distributed according to the terms
// of the GNU General Public License, incorporated herein by reference.

#include <stdio.h>
#include <string.h>

#include <FL/Fl.H>
#include <FL/Fl_Shared_Image.H>

#include "Fl_Epeg_Image.H"
#include "Slide.H"

int Slide::max_thumb_w;
int Slide::max_thumb_h;

Slide::Slide(int my_x, int my_y, int my_w, int my_h) : Fl_Toggle_Button(my_x, my_y, my_w, my_h) {
	file = NULL;
	color(FL_BLACK);
	down_color(FL_DARK_RED);
	labelsize(10);
	box(FL_PLASTIC_UP_BOX);
}

int
Slide::load(const char *f) {
	Fl_Shared_Image *im = Fl_Shared_Image::get(f);

	if (!im) {
		return 1;
	}

	file = strdup(f);
	label(file);
	image(im->copy());
	im->release();
	
	return 0;
}

int
Slide::reload() {
	if (file) {
		return load(file);
	} else {
		return 1;
	}
}

const char *
Slide::filename() {
	return file;
}

Fl_Image *
Slide::epeg_thumb_check(const char *name, uchar *header, int hdrlen) {
    if (hdrlen >= 4 &&
        memcmp(header, "\377\330\377", 3) == 0 &&
        header[3] >= 0xe0 && header[3] <= 0xef) {
        return new Fl_Epeg_Image(name, Slide::max_thumb_w, Slide::max_thumb_h);
    } else {
        return NULL;
    }
}

Fl_Image *
Slide::fallback_thumb_check(const char *name, uchar *header, int hdrlen) {
    static int again = 0;

    if (!again) {
        int im_w, im_h, thumb_w, thumb_h;

        again = 1; // Whoa what a hack !!!
        Fl_Shared_Image *tmp = Fl_Shared_Image::get(name);
        again = 0;

        if (!tmp) return NULL;

        im_w = tmp->w();
        im_h = tmp->h();

		if (im_w * Slide::max_thumb_h > im_h * Slide::max_thumb_w) {
			thumb_w = Slide::max_thumb_w;
			thumb_h = thumb_w * im_h / im_w;
		} else {
			thumb_h = Slide::max_thumb_h;
			thumb_w = thumb_h * im_w / im_h;
		}

        Fl_Image *ret = tmp->copy(thumb_w, thumb_h);
        tmp->release();
        return ret;
    } else {
        return NULL;
    }
}

int
Slide::handle(int event) {
	if (event == FL_PUSH) {
		if (Fl::event_button() != 1) {
			printf("Mouse%d\t%s\n", Fl::event_button(), file);
			fflush(stdout);
			return 1;
		}
	} else if (event == FL_DRAG) {
		if (Fl::event_button() != 1) {
			return 1;
		}
	}

	return Fl_Toggle_Button::handle(event);
}
	


void 
Slide::init(int _w, int _h) {
	Slide::max_thumb_w = _w;
	Slide::max_thumb_h = _h;
	Fl_Shared_Image::add_handler(epeg_thumb_check);
	Fl_Shared_Image::add_handler(fallback_thumb_check);
	fl_register_images();
}
